@@ -664,10 +664,15 @@ impl_xml_set (BonoboUIComponent  *component,
 	Bonobo_UIContainer_setNode (container, path, xml,
 				    name, real_ev);
 
-	if (BONOBO_EX (real_ev) && !ev)
+	if (BONOBO_EX (real_ev) && !ev) {
+		char *text;
+		
+		text = bonobo_exception_get_text (real_ev);
 		g_warning ("Serious exception on node_set '$%s' of '%s' to '%s'",
-			   bonobo_exception_get_text (real_ev), xml, path);
-
+			   text, xml, path);
+		g_free (text);
+	}
+	
 	if (!ev)
 		CORBA_exception_free (&tmp_ev);
 }
@@ -782,9 +787,14 @@ impl_xml_get (BonoboUIComponent *component,
 	xml = Bonobo_UIContainer_getNode (container, path, !recurse, real_ev);
 
 	if (BONOBO_EX (real_ev)) {
-		if (!ev)
+		if (!ev) {
+			char *text;
+			
+			text = bonobo_exception_get_text (real_ev);
 			g_warning ("Serious exception getting node '%s' '$%s'",
-				   path, bonobo_exception_get_text (real_ev));
+				   path, text);
+			g_free (text);
+		}
 
 		if (!ev)
 			CORBA_exception_free (&tmp_ev);
@@ -880,9 +890,14 @@ impl_xml_rm (BonoboUIComponent  *component,
 	Bonobo_UIContainer_removeNode (
 		container, path, priv->name, real_ev);
 
-	if (!ev && BONOBO_EX (real_ev))
+	if (!ev && BONOBO_EX (real_ev)) {
+		char *text;
+		
+		text = bonobo_exception_get_text (real_ev);
 		g_warning ("Serious exception removing path  '%s' '%s'",
-			   path, bonobo_exception_get_text (real_ev));
+			   path, text);
+		g_free (text);
+	}
 
 	if (!ev)
 		CORBA_exception_free (&tmp_ev);
@@ -924,9 +939,14 @@ bonobo_ui_component_object_set (BonoboUIComponent  *component,
 
 	Bonobo_UIContainer_setObject (container, path, control, real_ev);
 
-	if (!ev && BONOBO_EX (real_ev))
+	if (!ev && BONOBO_EX (real_ev)) {
+		char *text;
+		
+		text = bonobo_exception_get_text (real_ev);
 		g_warning ("Serious exception setting object '%s' '%s'",
-			   path, bonobo_exception_get_text (real_ev));
+			   path, text);
+		g_free (text);
+	}
 
 	if (!ev)
 		CORBA_exception_free (&tmp_ev);
@@ -970,9 +990,14 @@ bonobo_ui_component_object_get (BonoboUIComponent  *component,
 
 	ret = Bonobo_UIContainer_getObject (container, path, real_ev);
 
-	if (!ev && BONOBO_EX (real_ev))
+	if (!ev && BONOBO_EX (real_ev)) {
+		char *text;
+		
+		text = bonobo_exception_get_text (real_ev);
 		g_warning ("Serious exception getting object '%s' '%s'",
-			   path, bonobo_exception_get_text (real_ev));
+			   path, text);
+		g_free (text);
+	}
 
 	if (!ev)
 		CORBA_exception_free (&tmp_ev);
@@ -1066,9 +1091,13 @@ impl_freeze (BonoboUIComponent *component,
 
 	Bonobo_UIContainer_freeze (container, real_ev);
 
-	if (BONOBO_EX (real_ev) && !ev)
-		g_warning ("Serious exception on UI freeze '$%s'",
-			   bonobo_exception_get_text (real_ev));
+	if (BONOBO_EX (real_ev) && !ev) {
+		char *text;
+		
+		text = bonobo_exception_get_text (real_ev);
+		g_warning ("Serious exception on UI freeze '$%s'", text);
+		g_free (text);
+	}
 
 	if (!ev)
 		CORBA_exception_free (&tmp_ev);
@@ -1114,10 +1143,14 @@ impl_thaw (BonoboUIComponent *component,
 
 	Bonobo_UIContainer_thaw (container, real_ev);
 
-	if (BONOBO_EX (real_ev) && !ev)
-		g_warning ("Serious exception on UI thaw '$%s'",
-			   bonobo_exception_get_text (real_ev));
-
+	if (BONOBO_EX (real_ev) && !ev) {
+		char *text;
+		
+		text = bonobo_exception_get_text (real_ev);
+		g_warning ("Serious exception on UI thaw '$%s'", text);
+		g_free (text);
+	}
+	
 	if (!ev)
 		CORBA_exception_free (&tmp_ev);
 }
@@ -1305,9 +1338,13 @@ impl_exists (BonoboUIComponent *component,
 
 	if (BONOBO_EX (real_ev)) {
 		ret = FALSE;
-		if (!ev)
-			g_warning ("Serious exception on path_exists '$%s'",
-				   bonobo_exception_get_text (real_ev));
+		if (!ev) {
+			char *text;
+			
+			text = bonobo_exception_get_text (real_ev);
+			g_warning ("Serious exception on path_exists '$%s'", text);
+			g_free (text);
+		}
 	}
 
 	if (!ev)
@@ -1379,10 +1416,14 @@ bonobo_ui_component_unset_container (BonoboUIComponent *component)
 		Bonobo_UIContainer_removeNode (container, "/", name, &ev);
 		Bonobo_UIContainer_deregisterComponent (container, name, &ev);
 		
-		if (BONOBO_EX (&ev))
-			g_warning ("Serious exception deregistering component '%s'",
-				   bonobo_exception_get_text (&ev));
-
+		if (BONOBO_EX (&ev)) {
+			char *text;
+			
+			text = bonobo_exception_get_text (&ev);
+			g_warning ("Serious exception deregistering component '%s'", text);
+			g_free (text);
+		}
+		
 		CORBA_exception_free (&ev);
 
 		bonobo_object_release_unref (container, NULL);
@@ -1427,9 +1468,13 @@ bonobo_ui_component_set_container (BonoboUIComponent *component,
 		Bonobo_UIContainer_registerComponent (
 			ref_cont, name, corba_component, &ev);
 
-		if (BONOBO_EX (&ev))
-			g_warning ("Serious exception registering component '$%s'",
-				   bonobo_exception_get_text (&ev));
+		if (BONOBO_EX (&ev)) {
+			char *text;
+			
+			text = bonobo_exception_get_text (&ev);
+			g_warning ("Serious exception registering component '$%s'", text);
+			g_free (text);
+		}
 		
 		CORBA_exception_free (&ev);
 	} else
