@@ -946,6 +946,8 @@ state_update_new (BonoboUISync *sync,
 	sensitive = bonobo_ui_node_get_attr (node, "sensitive");
 	if (sensitive)
 		gtk_widget_set_sensitive (widget, atoi (sensitive));
+	else
+		gtk_widget_set_sensitive (widget, TRUE);
 	bonobo_ui_node_free_string (sensitive);
 
 	if ((state = bonobo_ui_node_get_attr (node, "state"))) {
@@ -1188,6 +1190,27 @@ bonobo_ui_engine_object_get (BonoboUIEngine    *engine,
 	return BONOBO_UI_ERROR_OK;
 }
 
+static char *
+get_parent_path (const char *path)
+{
+	int i, last_slash = 0;
+	char *ret;
+
+	for (i = 0; path [i]; i++) {
+		if (path [i] == '/')
+			last_slash = i;
+	}
+
+	if (!last_slash)
+		return NULL;
+
+	ret = g_malloc (last_slash + 1);
+	memcpy (ret, path, last_slash);
+	ret [last_slash] = '\0';
+
+	return ret;
+}
+
 /**
  * bonobo_ui_engine_xml_set_prop:
  * @engine: the engine
@@ -1208,20 +1231,26 @@ bonobo_ui_engine_xml_set_prop (BonoboUIEngine    *engine,
 			       const char        *value,
 			       const char        *component)
 {
-	BonoboUINode *node;
+	char *parent_path;
+	BonoboUINode *copy;
+	BonoboUINode *original;
 	
 	g_return_val_if_fail (BONOBO_IS_UI_ENGINE (engine), 
 			      BONOBO_UI_ERROR_BAD_PARAM);
 
-	node = bonobo_ui_engine_get_path (engine, path);
+	original = bonobo_ui_engine_get_path (engine, path);
 
-	if (!node) 
+	if (!original) 
 		return BONOBO_UI_ERROR_INVALID_PATH;
 
-	bonobo_ui_node_set_attr (node, property, value);
-	bonobo_ui_xml_set_dirty (engine->priv->tree, node);
-	
-	bonobo_ui_engine_update (engine);
+	copy = bonobo_ui_node_new (bonobo_ui_node_get_name (original));
+	bonobo_ui_node_copy_attrs (original, copy);
+	bonobo_ui_node_set_attr (copy, property, value);
+
+	parent_path = get_parent_path (path);
+	bonobo_ui_engine_xml_merge_tree (
+		engine, parent_path, copy, component);
+	g_free (parent_path);
 	
 	return BONOBO_UI_ERROR_OK;
 }
@@ -1523,6 +1552,31 @@ execute_state_updates (GSList *updates)
 	g_slist_free (updates);
 }
 
+static gboolean
+set_attr_changed (BonoboUINode *node,
+		  const char   *attr,
+		  const char   *value)
+{
+	gboolean changed = TRUE;
+
+	if (value) {
+		char *old_val;
+
+		old_val = bonobo_ui_node_get_attr (node, attr);
+
+		if (old_val && !strcmp (old_val, value))
+			changed = FALSE;
+
+		if (changed)
+			bonobo_ui_node_set_attr (node, attr, value);
+		
+		bonobo_ui_node_free_string (old_val);
+	} else
+		bonobo_ui_node_set_attr (node, attr, value);
+
+	return TRUE;
+}
+
 /*
  * set_cmd_attr:
  *   Syncs cmd / widgets on events [ event flag set ]
@@ -1547,8 +1601,8 @@ set_cmd_attr (BonoboUIEngine *engine,
 		NodeInfo *info = bonobo_ui_xml_get_data (
 			engine->priv->tree, node);
 
-		bonobo_ui_node_set_attr (node, prop, value);
-		state_update_now (engine, node, info->widget);
+		if (set_attr_changed (node, prop, value))
+			state_update_now (engine, node, info->widget);
 		return;
 	}
 
@@ -1558,26 +1612,26 @@ set_cmd_attr (BonoboUIEngine *engine,
 		 prop, value);
 #endif
 
-	bonobo_ui_node_set_attr (cmd_node, prop, value);
-
-	if (event) {
-		GSList *updates;
-		char   *cmd_name = bonobo_ui_node_get_attr (
-			cmd_node, "name");
+	if (set_attr_changed (cmd_node, prop, value)) {
+		if (event) {
+			GSList *updates;
+			char   *cmd_name = bonobo_ui_node_get_attr (
+				cmd_node, "name");
 
 
-		updates = make_updates_for_command (
-			engine, NULL, cmd_node, cmd_name);
+			updates = make_updates_for_command (
+				engine, NULL, cmd_node, cmd_name);
 
-		execute_state_updates (updates);
+			execute_state_updates (updates);
 
-		bonobo_ui_node_free_string (cmd_name);
-	} else {
-		BonoboUIXmlData *data =
-			bonobo_ui_xml_get_data (
-				engine->priv->tree, cmd_node);
+			bonobo_ui_node_free_string (cmd_name);
+		} else {
+			BonoboUIXmlData *data =
+				bonobo_ui_xml_get_data (
+					engine->priv->tree, cmd_node);
 
-		data->dirty = TRUE;
+			data->dirty = TRUE;
+		}
 	}
 }
 
